/**
@authors: Daniel DAVIDS, Frederic BERNARD, EN-ICE SCD
@date: 12/08/2009
@file unLHCCircuit_xml.ctl

Modification History:
  30/11/2011: Axel
  - CIR-40: fwXml function not defined
*/

/**
* @name LHCCircuit functions to read the CRYO XML file
* @{
*/

//--------------------------------------------------------------------------------------------------------------------------------

// unLHCCircuit_xml_getData
/**
Purpose:
get CRYO devices list and the correponding Circuit information

	@param docum: int, input:  the document identifier
	@param ident: int, input:  the node identifier
	@param system_name: string, input: PVSS Sytem Name
	@param dsDevices: dyn_string, output: list of CRYO devices
	@param ddsCircuits: dyn_dyn_string, output: for each CRYO device, list of circuit information
	@param exInfo: dyn_string, output: exception are returned here
        
Usage: Public

PVSS manager usage: UI and Ctrl

Constraints:
	. PVSS version: 3.6 + Xml patch 
	. operating system: WXP, SLC4
	. distributed system: yes.
*/

void unLHCCircuit_xml_getData (unsigned docum, unsigned ident, string system_name, dyn_string &dsDevices, dyn_dyn_string &ddsCircuits, dyn_string &exInfo )
{
dyn_string dsCircuit;
string thecircuit, sInterlock, sCircuit, sType, sPw;
int nbofcircuits, iDev, ident_init;
dyn_uint nodes, deviceNodes, devicesSubNodes;
int rtn_code;
  
  ident_init = xmlFirstChild(docum);
  rtn_code = xmlChildNodes(docum, ident_init, nodes);
  rtn_code = xmlChildNodes(docum, ident, nodes);    

  if ( rtn_code != 0 )
  {
    DebugN("unLHCCircuit_xml_getData() - xmlChildNodes Fails with Return-Code '"+rtn_code+"'");
    return;
  }
        
  nbofcircuits = dynlen(nodes);
  //DebugN("unLHCCircuit_xml_getData() - Total Number of Circuits: " + nbofcircuits + " - System: " + system_name);

  for ( int idx = 1 ; idx <= nbofcircuits ; ++idx )
  {
    thecircuit = xmlNodeName ( docum , nodes[idx] );
    
    xmlGetElementAttribute( docum, nodes[idx], "name" , sCircuit);    
    xmlGetElementAttribute( docum, nodes[idx], "interlock_type" , sInterlock);
    xmlGetElementAttribute( docum, nodes[idx], "circuit_type" , sType);
    xmlGetElementAttribute( docum, nodes[idx], "powering_subsector", sPw);            
    dsCircuit=makeDynString(sCircuit,sInterlock,sType,sPw);
    
    rtn_code = xmlChildNodes(docum, nodes[idx], deviceNodes);
    if(rtn_code != 0) {
      DebugN("unLHCCircuit_xml_getData() - xmlChildNodes Fails with Return-Code '"+rtn_code+"'");
      return;
    }
    
    for(int i=1 ; i <= dynlen(deviceNodes) ; i++) {
      iDev++;
      ddsCircuits[iDev] = dsCircuit;
      
      if(xmlNodeType(docum, deviceNodes[i]) != XML_ELEMENT_NODE) {
        DebugN("unLHCCircuit_xml_getData() - an element node was expected for node " + i + " of circuit " + sCircuit);
        return;
      }
      
      rtn_code = xmlChildNodes(docum, deviceNodes[i], devicesSubNodes);
      if(rtn_code != 0) {
        DebugN("unLHCCircuit_xml_getData() - xmlChildNodes Fails with Return-Code '"+rtn_code+"'");
        return;
      }
      
      if(dynlen(devicesSubNodes) != 1) {
        DebugN("unLHCCircuit_xml_getData() - xmlChildNodes Fails to returning exactly one node");
        return;
      }
      
      if(xmlNodeType(docum, devicesSubNodes[1]) != XML_TEXT_NODE) {
        DebugN("unLHCCircuit_xml_getData() - a text node was expected for the subnode of node " + i + " of circuit " + sCircuit);
        return;
      }
      
      dynAppend(dsDevices, xmlNodeValue(docum, devicesSubNodes[1]));
    }
  }
}

//--------------------------------------------------------------------------------------------------------------------------------

// unLHCCircuit_xml_getAllSystems
/**
Purpose:
Get the list of all the systems 

	@param docum: int, input:  the document identifier
	@param ident: int, input:  the node identifier
	@param mSystemId: mapping, output: relation between the System Name and the Node identifier
	@param exInfo: dyn_string, output: exception are returned here

Usage: Public

PVSS manager usage: UI and Ctrl

Constraints:
	. PVSS version: 3.6 + Xml patch 
	. operating system: WXP, SLC4
	. distributed system: yes.
*/

void unLHCCircuit_xml_getAllSystems(int docum ,int ident ,mapping &mSystemId ,dyn_string &exInfo)
{
string system_name;
int nbofsystems;
dyn_uint nodes;
int rtn_code;
  
  rtn_code = xmlChildNodes ( docum , ident , nodes );
  
  if ( rtn_code != 0 )
  {
    DebugN ("unLHCCircuit_xml_getAllSystems() - xmlChildNodes Fails with Return-Code '"+rtn_code+"'" );
    return;
  }
        
  nbofsystems = dynlen(nodes);
  //DebugN("unLHCCircuit_xml_getAllSystems() - Total Number of Systems: " + nbofsystems);
                                  
  for ( int idx = 1 ; idx <= nbofsystems ; ++idx )
  {
    xmlGetElementAttribute(docum,nodes[idx],"name",system_name);    
    //DebugN("unLHCCircuit_xml_getAllSystems() - System: '"+system_name+"'");
    mSystemId[system_name]=nodes[idx];
  }
}

//--------------------------------------------------------------------------------------------------------------------------------

// unLHCCircuit_xml_init
/**
Purpose:
Initialize the tree device overview 

	@param xml_full_name: string ,input, the file path

Usage: Public

PVSS manager usage: UI and Ctrl

Constraints:
	. PVSS version: 3.6 + Xml patch 
	. operating system: WXP, SLC4
	. distributed system: yes.
*/

int unLHCCircuit_xml_init(const string xml_full_name)
{
int docum;
string errMsg, errLin, errCol;

 if(xml_full_name!="")
    {    
    docum = xmlDocumentFromFile ( xml_full_name , errMsg , errLin , errCol );
    // DebugN ( "document = " + docum );
  
    if (docum < 0 )
      DebugN ( "unLHCCircuit_xml_init() - Parsing Error-Message = '" + errMsg + "' Line=" + errLin + " Column=" + errCol );
    else
      DebugN ("unLHCCircuit_xml_init() - Parsing OK - " + xml_full_name);    
    }
  else
    {
    docum=-10;
    DebugN ("unLHCCircuit_xml_init() - No XML document defined!");
    }
  
  return docum;
}

//--------------------------------------------------------------------------------------------------------------------------------

// unLHCCircuit_xml_getSystemMapping
/**
Purpose:
Initialize the mapping variable with System name Id 

	@param docum: int ,input, Doc Id
	@param mSystemId: mapping, output: relation between the System Name and the Node identifier

Usage: Public

PVSS manager usage: UI and Ctrl

Constraints:
	. PVSS version: 3.6 + Xml patch 
	. operating system: WXP, SLC4
	. distributed system: yes.
*/

void unLHCCircuit_xml_getSystemMapping(const int docum, mapping &mSystemId)
{
int rtn_code, ident;
string errMsg, errLin, errCol;
string element, system_name;
mapping attribs;
dyn_string exInfo, dsDevices;
dyn_dyn_string ddsCircuits;
  
  if (docum < 0 )
    {
    DebugN ( "unLHCCircuit_xml_getSystemMapping() - No XML document defined!");
    }
  else
    {
    ident = xmlFirstChild ( docum );
    // DebugN ( "1-child = " + ident );
    
    element = xmlNodeName ( docum , ident );
    attribs = xmlElementAttributes ( docum , ident );  
    unLHCCircuit_xml_getAllSystems( docum, ident, mSystemId, exInfo);
    DebugN ("XmlFile: " + element + " - Type: " + attribs["type"] + " - Conf: " + attribs["config"] + " - Date: " + attribs["date"] );
    }
}

//--------------------------------------------------------------------------------------------------------------------------------

// unLHCCircuit_xml_close
/**
Purpose:
Close the XML file

	@param docum: int, input:  the document identifier

Usage: Public

PVSS manager usage: UI and Ctrl

Constraints:
	. PVSS version: 3.6 + Xml patch 
	. operating system: WXP, SLC4
	. distributed system: yes.
*/

int unLHCCircuit_xml_close(int docum)
{
int iRes;

  if (docum < 0 )
    DebugN("unLHCCircuit_xml_close() - No XML document defined!");
  else 
    iRes = xmlCloseDocument(docum);
  
  return iRes;
}

